@@ -1,42 +1,43 @@
-from . import config
+from . import config  # isort:skip; load distributed configuration first
 import dask
 from dask.config import config
+from dask.utils import import_required
+
+from ._version import get_versions
 from .actor import Actor, ActorFuture
-from .core import connect, rpc, Status
-from .deploy import LocalCluster, Adaptive, SpecCluster, SSHCluster
-from .diagnostics.progressbar import progress
-from .diagnostics.plugin import WorkerPlugin, SchedulerPlugin, PipInstall
 from .client import (
     Client,
-    Executor,
     CompatibleExecutor,
-    wait,
+    Executor,
+    Future,
     as_completed,
     default_client,
     fire_and_forget,
-    Future,
     futures_of,
+    get_task_metadata,
     get_task_stream,
     performance_report,
-    get_task_metadata,
+    wait,
 )
+from .core import Status, connect, rpc
+from .deploy import Adaptive, LocalCluster, SpecCluster, SSHCluster
+from .diagnostics.plugin import PipInstall, SchedulerPlugin, WorkerPlugin
+from .diagnostics.progressbar import progress
+from .event import Event
 from .lock import Lock
 from .multi_lock import MultiLock
 from .nanny import Nanny
 from .pubsub import Pub, Sub
 from .queues import Queue
+from .scheduler import Scheduler
 from .security import Security
 from .semaphore import Semaphore
-from .event import Event
-from .scheduler import Scheduler
 from .threadpoolexecutor import rejoin
-from .utils import sync, TimeoutError, CancelledError
+from .utils import CancelledError, TimeoutError, sync
 from .variable import Variable
-from .worker import Worker, get_worker, get_client, secede, Reschedule
+from .worker import Reschedule, Worker, get_client, get_worker, secede
 from .worker_client import local_client, worker_client
 
-from ._version import get_versions
-
 versions = get_versions()
 __version__ = versions["version"]
 __git_revision__ = versions["full-revisionid"]
@@ -45,6 +46,15 @@ del get_versions, versions
 if dask.config.get("distributed.admin.event-loop") in ("asyncio", "tornado"):
     pass
 elif dask.config.get("distributed.admin.event-loop") == "uvloop":
+    import_required(
+        "uvloop",
+        "The distributed.admin.event-loop configuration value "
+        "is set to 'uvloop' but the uvloop module is not installed"
+        "\n\n"
+        "Please either change the config value or install one of the following\n"
+        "    conda install uvloop\n"
+        "    pip install uvloop",
+    )
     import uvloop
 
     uvloop.install()
