@@ -1,19 +1,17 @@
-import atexit
+import asyncio
 import logging
 import os
-from queue import Queue as PyQueue
 import re
 import threading
 import weakref
-import asyncio
-import dask
+from queue import Queue as PyQueue
 
-from .utils import mp_context, TimeoutError
-
-from tornado import gen
 from tornado.concurrent import Future
 from tornado.ioloop import IOLoop
 
+import dask
+
+from .utils import TimeoutError, mp_context
 
 logger = logging.getLogger(__name__)
 
@@ -40,13 +38,13 @@ def _call_and_set_future(loop, future, func, *args, **kwargs):
         _loop_add_callback(loop, future.set_result, res)
 
 
-class _ProcessState(object):
+class _ProcessState:
     is_alive = False
     pid = None
     exitcode = None
 
 
-class AsyncProcess(object):
+class AsyncProcess:
     """
     A coroutine-compatible multiprocessing.Process-alike.
     All normally blocking methods are wrapped in Tornado coroutines.
@@ -81,8 +79,10 @@ class AsyncProcess(object):
                 dask.config.global_config,
             ),
         )
-        _dangling.add(self._process)
         self._name = self._process.name
+        self._proc_finalizer = weakref.finalize(
+            self, _asyncprocess_finalizer, self._process
+        )
         self._watch_q = PyQueue()
         self._exit_future = Future()
         self._exit_callback = None
@@ -118,8 +118,8 @@ class AsyncProcess(object):
             # We don't join the thread here as a finalizer can be called
             # asynchronously from anywhere
 
-        self._finalizer = weakref.finalize(self, stop_thread, q=self._watch_q)
-        self._finalizer.atexit = False
+        self._thread_finalizer = weakref.finalize(self, stop_thread, q=self._watch_q)
+        self._thread_finalizer.atexit = False
 
     def _on_exit(self, exitcode):
         # Called from the event loop when the child process exited
@@ -162,7 +162,7 @@ class AsyncProcess(object):
 
     @staticmethod
     def reset_logger_locks():
-        """ Python 2's logger's locks don't survive a fork event
+        """Python 2's logger's locks don't survive a fork event
 
         https://github.com/dask/distributed/issues/1491
         """
@@ -267,8 +267,7 @@ class AsyncProcess(object):
         self._watch_q.put_nowait({"op": "terminate", "future": fut})
         return fut
 
-    @gen.coroutine
-    def join(self, timeout=None):
+    async def join(self, timeout=None):
         """
         Wait for the child process to exit.
 
@@ -279,10 +278,12 @@ class AsyncProcess(object):
         if self._state.exitcode is not None:
             return
         if timeout is None:
-            yield self._exit_future
+            await self._exit_future
         else:
             try:
-                yield asyncio.wait_for(self._exit_future, timeout)
+                # Shield otherwise the timeout cancels the future and our
+                # on_exit callback will try to set a result on a canceled future
+                await asyncio.wait_for(asyncio.shield(self._exit_future), timeout)
             except TimeoutError:
                 pass
 
@@ -292,7 +293,7 @@ class AsyncProcess(object):
         immediately and does not ensure the child process has exited.
         """
         if not self._closed:
-            self._finalizer()
+            self._thread_finalizer()
             self._process = None
             self._closed = True
 
@@ -334,15 +335,10 @@ class AsyncProcess(object):
         self._process.daemon = value
 
 
-_dangling = weakref.WeakSet()
-
-
-@atexit.register
-def _cleanup_dangling():
-    for proc in list(_dangling):
-        if proc.is_alive():
-            try:
-                logger.info("reaping stray process %s" % (proc,))
-                proc.terminate()
-            except OSError:
-                pass
+def _asyncprocess_finalizer(proc):
+    if proc.is_alive():
+        try:
+            logger.info("reaping stray process %s" % (proc,))
+            proc.terminate()
+        except OSError:
+            pass
