@@ -1,9 +1,6 @@
-from __future__ import print_function, division, absolute_import
-
 import pytest
 
 from distributed.counter import Counter
-from distributed.utils_test import loop  # noqa F401
 
 try:
     from distributed.counter import Digest
