@@ -42,6 +42,7 @@ def test_defaults():
     assert sec.tls_scheduler_cert is None
     assert sec.tls_worker_key is None
     assert sec.tls_worker_cert is None
+    assert sec.extra_conn_args == {}
 
 
 def test_constructor_errors():
@@ -92,7 +93,10 @@ def test_kwargs():
     }
     with dask.config.set(c):
         sec = Security(
-            tls_scheduler_cert="newcert.pem", require_encryption=True, tls_ca_file=None
+            tls_scheduler_cert="newcert.pem",
+            require_encryption=True,
+            tls_ca_file=None,
+            extra_conn_args={"headers": {"Auth": "Token abc"}},
         )
     assert sec.require_encryption is True
     assert sec.tls_ca_file is None
@@ -103,6 +107,7 @@ def test_kwargs():
     assert sec.tls_scheduler_cert == "newcert.pem"
     assert sec.tls_worker_key is None
     assert sec.tls_worker_cert is None
+    assert sec.extra_conn_args == {"headers": {"Auth": "Token abc"}}
 
 
 def test_repr():
@@ -200,6 +205,30 @@ def test_connection_args():
     assert len(tls_13_ciphers) in (0, 3)
 
 
+def test_extra_conn_args_connection_args():
+    c = {
+        "distributed.comm.tls.ca-file": ca_file,
+        "distributed.comm.tls.scheduler.key": key1,
+        "distributed.comm.tls.scheduler.cert": cert1,
+        "distributed.comm.tls.worker.cert": keycert1,
+    }
+    with dask.config.set(c):
+        sec = Security(extra_conn_args={"headers": {"Authorization": "Token abcd"}})
+
+    d = sec.get_connection_args("scheduler")
+    assert not d["require_encryption"]
+    assert d["extra_conn_args"]["headers"] == {"Authorization": "Token abcd"}
+    ctx = d["ssl_context"]
+
+    d = sec.get_connection_args("worker")
+    assert d["extra_conn_args"]["headers"] == {"Authorization": "Token abcd"}
+
+    # No cert defined => no TLS
+    d = sec.get_connection_args("client")
+    assert d.get("ssl_context") is None
+    assert d["extra_conn_args"]["headers"] == {"Authorization": "Token abcd"}
+
+
 def test_listen_args():
     def basic_checks(ctx):
         assert ctx.verify_mode == ssl.CERT_REQUIRED
@@ -376,6 +405,13 @@ def test_temporary_credentials():
         assert val not in sec_repr
 
 
+def test_extra_conn_args_in_temporary_credentials():
+    pytest.importorskip("cryptography")
+
+    sec = Security.temporary(extra_conn_args={"headers": {"X-Request-ID": "abcd"}})
+    assert sec.extra_conn_args == {"headers": {"X-Request-ID": "abcd"}}
+
+
 @pytest.mark.asyncio
 async def test_tls_temporary_credentials_functional():
     pytest.importorskip("cryptography")
